'use strict';

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = reducer;
exports.updateUI = updateUI;
exports.massUpdateUI = massUpdateUI;
exports.setDefaultUI = setDefaultUI;
exports.unmountUI = unmountUI;
exports.mountUI = mountUI;
exports.reducerEnhancer = exports.defaultState = exports.SET_DEFAULT_UI_STATE = exports.UPDATE_UI_STATE = exports.MASS_UPDATE_UI_STATE = void 0;

var _values = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/values"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/array/is-array"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/objectSpread"));

var _invariant = _interopRequireDefault(require("invariant"));

var _utils = require("./utils");

// For updating multiple UI variables at once.  Each variable might be part of
// a different context; this means that we need to either call updateUI on each
// key of the object to update or do transformations within one action in the
// reducer. The latter only triggers one store change event and is more
// performant.
var MASS_UPDATE_UI_STATE = '@@react-redux-ui-tools/MASS_UPDATE_UI_STATE';
exports.MASS_UPDATE_UI_STATE = MASS_UPDATE_UI_STATE;
var UPDATE_UI_STATE = '@@react-redux-ui-tools/UPDATE_UI_STATE';
exports.UPDATE_UI_STATE = UPDATE_UI_STATE;
var SET_DEFAULT_UI_STATE = '@@react-redux-ui-tools/SET_DEFAULT_UI_STATE'; // These are private consts used in actions only given to the UI decorator.

exports.SET_DEFAULT_UI_STATE = SET_DEFAULT_UI_STATE;
var MOUNT_UI_STATE = '@@react-redux-ui-tools/MOUNT_UI_STATE';
var UNMOUNT_UI_STATE = '@@react-redux-ui-tools/UNMOUNT_UI_STATE';
var defaultState = {
  __reducers: {// This contains a map of component paths (joined by '.') to an object
    // containing the fully qualified path and the reducer function:
    // 'parent.child': {
    //   path: ['parent', 'child'],
    //   func: (state, action) => { ... }
    // }
  }
};
exports.defaultState = defaultState;

function reducer(state, action) {
  var mapImplementation = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Object;

  if (typeof state === 'undefined') {
    if (mapImplementation === Object) {
      state = defaultState;
    } else {
      state = new mapImplementation((0, _objectSpread2["default"])({}, defaultState, {
        __reducers: new mapImplementation((0, _objectSpread2["default"])({}, defaultState.__reducers))
      }));
    }
  }

  var key = action.payload && (action.payload.key || []);

  if (!(0, _isArray["default"])(key)) {
    key = [key];
  }

  switch (action.type) {
    case UPDATE_UI_STATE:
      var _action$payload = action.payload,
          name = _action$payload.name,
          value = _action$payload.value;

      if (typeof value === 'function') {
        state = (0, _utils.updateIn)(state, key.concat(name), value);
      } else {
        state = (0, _utils.setIn)(state, key.concat(name), value);
      }

      break;

    case MASS_UPDATE_UI_STATE:
      var _action$payload2 = action.payload,
          uiVars = _action$payload2.uiVars,
          transforms = _action$payload2.transforms;
      state = (0, _utils.withMutations)(state, function (s) {
        (0, _keys["default"])(transforms).forEach(function (k) {
          var path = uiVars[k];
          (0, _invariant["default"])(path, "Couldn't find variable ".concat(k, " within your component's UI state ") + "context. Define ".concat(k, " before using it in the @ui decorator"));
          s.setIn(path.concat(k), transforms[k]);
        });
      });
      break;

    case SET_DEFAULT_UI_STATE:
      // Replace all UI under a key with the given values
      state = (0, _utils.setIn)(state, key, mapImplementation === Object ? action.payload.value : new mapImplementation(action.payload.value));
      break;

    case MOUNT_UI_STATE:
      var _action$payload3 = action.payload,
          defaults = _action$payload3.defaults,
          customReducer = _action$payload3.customReducer;
      state = (0, _utils.withMutations)(state, function (s) {
        // Set the defaults for the component
        s.setIn(key, mapImplementation === Object ? defaults : new mapImplementation(defaults)); // If this component has a custom reducer add it to the list.
        // We store the reducer func and UI path for the current component
        // inside the __reducers map.

        if (customReducer) {
          var path = key.join('.');
          s.setIn(['__reducers', path], {
            path: key,
            func: customReducer
          });
        }

        return s;
      });
      break;

    case UNMOUNT_UI_STATE:
      // We have to use deleteIn as react unmounts root components first;
      // this means that using setIn in child contexts will fail as the root
      // context will be stored as undefined in our state
      state = (0, _utils.withMutations)(state, function (s) {
        s.deleteIn(key); // Remove any custom reducers

        s.deleteIn(['__reducers', key.join('.')]);
      });
      break;
  }

  var customReducers = (0, _utils.get)(state, '__reducers');

  if ((mapImplementation === Object ? (0, _keys["default"])(customReducers).length : customReducers.size) > 0) {
    state = (0, _utils.withMutations)(state, function (mut) {
      (mapImplementation === Object ? (0, _values["default"])(customReducers) : customReducers).forEach(function (r) {
        // This calls each custom reducer with the UI state for each custom
        // reducer with the component's UI state tree passed into it.
        //
        // NOTE: Each component's reducer gets its own UI state: not the entire
        // UI reducer's state. Whatever is returned from this reducer is set
        // within the **components** UI scope.
        //
        // This is because it's the only way to update UI state for components
        // without keys - you need to know the path in advance to update state
        // from a reducer.  If you have list of components with no UI keys in
        // the component heirarchy, any children will not be able to use custom
        // reducers as the path is random.
        //
        // TODO: Potentially add the possibility for a global UI state reducer?
        //       Though why wouldn't you just add a custom reducer to the
        //       top-level component?
        var path = r.path,
            func = r.func;
        var newState = func(mut.getIn(path), action);

        if (newState === undefined) {
          throw new Error("Your custom UI reducer at path ".concat(path.join('.'), " must return some state"));
        }

        mut.setIn(path, newState);
      });
      return mut;
    });
  }

  return state;
}

var reducerEnhancer = function reducerEnhancer(customReducer, mapImplementation) {
  return function (state, action) {
    state = reducer(state, action, mapImplementation);

    if (typeof customReducer === 'function') {
      state = customReducer(state, action);
    }

    return state;
  };
};

exports.reducerEnhancer = reducerEnhancer;

function updateUI(key, name, value) {
  return {
    type: UPDATE_UI_STATE,
    payload: {
      key: key,
      name: name,
      value: value
    }
  };
}

;

function massUpdateUI(uiVars, transforms) {
  return {
    type: MASS_UPDATE_UI_STATE,
    payload: {
      uiVars: uiVars,
      transforms: transforms
    }
  };
} // Exposed to components, allowing them to reset their and all child scopes to
// the default variables set up


function setDefaultUI(key, value) {
  return {
    type: SET_DEFAULT_UI_STATE,
    payload: {
      key: key,
      value: value
    }
  };
}

;
/** Private, decorator only actions **/
// This is not exposed to your components; it's only used in the decorator.

function unmountUI(key) {
  return {
    type: UNMOUNT_UI_STATE,
    payload: {
      key: key
    }
  };
}

;
/**
 * Given the key/path, set of defaults and custom reducer for a UI component
 * during construction prepare the state of the UI reducer
 *
 */

function mountUI(key, defaults, customReducer) {
  return {
    type: MOUNT_UI_STATE,
    payload: {
      key: key,
      defaults: defaults,
      customReducer: customReducer
    }
  };
}